#pragma once

#include <string>
#include <optional>
#include <filesystem>

#include <curl/curl.h>
#include <stdio.h>

class Dependency {
public:
    enum class Type {
        automatic,
        simple,
        mason
    };

    Dependency(
        const std::string& path,
        Type type = Type::automatic,
        const std::optional<std::string>& name = std::nullopt,
        const std::optional<std::string>& version = std::nullopt
    );

    Type getType() const;
    std::optional<std::string> getName() const;
    std::optional<std::string> getVersion() const;

    bool prepare(const std::filesystem::path& source, const std::filesystem::path& destination);

    const std::string path;

private:
    static size_t write(void *file, size_t size, size_t nmemb, void *stream);
    bool download(const std::filesystem::path& destination);

private:
    Type type;
    std::optional<std::string> name;
    std::optional<std::string> version;
};
